require 'strict'
local bit = require 'bit'
local ffi = require 'ffi'

local C = ffi.C

ffi.cdef[[
union htonll_u {
    double d;
    uint64_t li;
    uint32_t i[2];
};
]]

local ec = {} -- endian conversion
local htonll_data = ffi.new('union htonll_u')
if ffi.abi("be") then -- nothing to do
    function ec.htonl(b) return b end
    function ec.htons(b) return b end
    function ec.htonll(b) return b end
    function ec.htonll_double(b) return b end
else
    function ec.htonl(b) return bit.bswap(b) end
    function ec.htons(b) return bit.rshift(bit.bswap(b), 16) end
    function ec.htonll_double(b)    -- assumes b is ffi C type double
        htonll_data.d = b
        local i1 = ec.htonl(htonll_data.i[1])
        local i0 = ec.htonl(htonll_data.i[0])
        htonll_data.i[1] = i0
        htonll_data.i[0] = i1
        return htonll_data.d
    end
    function ec.htonll(b)    -- assumes b is ffi C type long long
        htonll_data.li = b
        local i1 = ec.htonl(htonll_data.i[1])
        local i0 = ec.htonl(htonll_data.i[0])
        htonll_data.i[1] = i0
        htonll_data.i[0] = i1
        return htonll_data.li
    end
end
ec.ntohl = ec.htonl -- reverse is the same
ec.ntohs = ec.htons -- reverse is the same
ec.ntohll = ec.htonll -- reverse is the same
ec.ntohll_double = ec.htonll_double -- reverse is the same

return ec
