local ffi = require 'ffi'
local lj_include = require 'lj_include'

ffi.cdef[[
typedef struct {
    unsigned int clock_rate;
    unsigned int clock_type;
    unsigned short loopback;
} sync_serial_settings;
typedef struct {
    unsigned int clock_rate;
    unsigned int clock_type;
    unsigned short loopback;
    unsigned int slot_map;
} te1_settings;
typedef struct {
    unsigned short encoding;
    unsigned short parity;
} raw_hdlc_proto;
typedef struct {
    unsigned int t391;
    unsigned int t392;
    unsigned int n391;
    unsigned int n392;
    unsigned int n393;
    unsigned short lmi;
    unsigned short dce;
} fr_proto;
typedef struct {
    unsigned int dlci;
} fr_proto_pvc;
typedef struct {
    unsigned int dlci;
    char master[16];
} fr_proto_pvc_info;
typedef struct {
    unsigned int interval;
    unsigned int timeout;
} cisco_proto;
]]

local defs = {
    GENERIC_HDLC_VERSION = '4',
    CLOCK_DEFAULT = '0',
    CLOCK_EXT = '1',
    CLOCK_INT = '2',
    CLOCK_TXINT = '3',
    CLOCK_TXFROMRX = '4',
    ENCODING_DEFAULT = '0',
    ENCODING_NRZ = '1',
    ENCODING_NRZI = '2',
    ENCODING_FM_MARK = '3',
    ENCODING_FM_SPACE = '4',
    ENCODING_MANCHESTER = '5',
    PARITY_DEFAULT = '0',
    PARITY_NONE = '1',
    PARITY_CRC16_PR0 = '2',
    PARITY_CRC16_PR1 = '3',
    PARITY_CRC16_PR0_CCITT = '4',
    PARITY_CRC16_PR1_CCITT = '5',
    PARITY_CRC32_PR0_CCITT = '6',
    PARITY_CRC32_PR1_CCITT = '7',
    LMI_DEFAULT = '0',
    LMI_NONE = '1',
    LMI_ANSI = '2',
    LMI_CCITT = '3',
    LMI_CISCO = '4',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

