local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('linux_types')
lj_include('linux_socket')
lj_include('linux_hdlc_ioctl')

ffi.cdef[[
enum net_device_flags {
    IFF_UP = 1 << 0,
    IFF_BROADCAST = 1 << 1,
    IFF_DEBUG = 1 << 2,
    IFF_LOOPBACK = 1 << 3,
    IFF_POINTOPOINT = 1 << 4,
    IFF_NOTRAILERS = 1 << 5,
    IFF_RUNNING = 1 << 6,
    IFF_NOARP = 1 << 7,
    IFF_PROMISC = 1 << 8,
    IFF_ALLMULTI = 1 << 9,
    IFF_MASTER = 1 << 10,
    IFF_SLAVE = 1 << 11,
    IFF_MULTICAST = 1 << 12,
    IFF_PORTSEL = 1 << 13,
    IFF_AUTOMEDIA = 1 << 14,
    IFF_DYNAMIC = 1 << 15,
    IFF_LOWER_UP = 1 << 16,
    IFF_DORMANT = 1 << 17,
    IFF_ECHO = 1 << 18,
};
enum {
    IF_OPER_UNKNOWN,
    IF_OPER_NOTPRESENT,
    IF_OPER_DOWN,
    IF_OPER_LOWERLAYERDOWN,
    IF_OPER_TESTING,
    IF_OPER_DORMANT,
    IF_OPER_UP,
};
enum {
    IF_LINK_MODE_DEFAULT,
    IF_LINK_MODE_DORMANT,
};
struct ifmap {
    unsigned long mem_start;
    unsigned long mem_end;
    unsigned short base_addr;
    unsigned char irq;
    unsigned char dma;
    unsigned char port;
};
struct if_settings {
    unsigned int type;
    unsigned int size;
    union {
	raw_hdlc_proto *raw_hdlc;
	cisco_proto *cisco;
	fr_proto *fr;
	fr_proto_pvc *fr_pvc;
	fr_proto_pvc_info *fr_pvc_info;
	sync_serial_settings *sync;
	te1_settings *te1;
    } ifs_ifsu;
};
struct ifreq {
    union {
	char ifrn_name[16];
    } ifr_ifrn;
    union {
	struct sockaddr ifru_addr;
	struct sockaddr ifru_dstaddr;
	struct sockaddr ifru_broadaddr;
	struct sockaddr ifru_netmask;
	struct sockaddr ifru_hwaddr;
	short ifru_flags;
	int ifru_ivalue;
	int ifru_mtu;
	struct ifmap ifru_map;
	char ifru_slave[16];
	char ifru_newname[16];
	void *ifru_data;
	struct if_settings ifru_settings;
    } ifr_ifru;
};
struct ifconf {
    int ifc_len;
    union {
	char *ifcu_buf;
	struct ifreq *ifcu_req;
    } ifc_ifcu;
};
]]

local defs = {
    IFNAMSIZ = '16',
    IFALIASZ = '256',
    IF_GET_IFACE = '0x0001',
    IF_GET_PROTO = '0x0002',
    IF_IFACE_V35 = '0x1000',
    IF_IFACE_V24 = '0x1001',
    IF_IFACE_X21 = '0x1002',
    IF_IFACE_T1 = '0x1003',
    IF_IFACE_E1 = '0x1004',
    IF_IFACE_SYNC_SERIAL = '0x1005',
    IF_IFACE_X21D = '0x1006',
    IF_PROTO_HDLC = '0x2000',
    IF_PROTO_PPP = '0x2001',
    IF_PROTO_CISCO = '0x2002',
    IF_PROTO_FR = '0x2003',
    IF_PROTO_FR_ADD_PVC = '0x2004',
    IF_PROTO_FR_DEL_PVC = '0x2005',
    IF_PROTO_X25 = '0x2006',
    IF_PROTO_HDLC_ETH = '0x2007',
    IF_PROTO_FR_ADD_ETH_PVC = '0x2008',
    IF_PROTO_FR_DEL_ETH_PVC = '0x2009',
    IF_PROTO_FR_PVC = '0x200A',
    IF_PROTO_FR_ETH_PVC = '0x200B',
    IF_PROTO_RAW = '0x200C',
    IFHWADDRLEN = '6',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

