local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('linux_types')
lj_include('linux_ethtool')

ffi.cdef[[
struct mii_ioctl_data {
    __u16 phy_id;
    __u16 reg_num;
    __u16 val_in;
    __u16 val_out;
};
]]

local defs = {
    MII_BMCR = '0x00',
    MII_BMSR = '0x01',
    MII_PHYSID1 = '0x02',
    MII_PHYSID2 = '0x03',
    MII_ADVERTISE = '0x04',
    MII_LPA = '0x05',
    MII_EXPANSION = '0x06',
    MII_CTRL1000 = '0x09',
    MII_STAT1000 = '0x0a',
    MII_MMD_CTRL = '0x0d',
    MII_MMD_DATA = '0x0e',
    MII_ESTATUS = '0x0f',
    MII_DCOUNTER = '0x12',
    MII_FCSCOUNTER = '0x13',
    MII_NWAYTEST = '0x14',
    MII_RERRCOUNTER = '0x15',
    MII_SREVISION = '0x16',
    MII_RESV1 = '0x17',
    MII_LBRERROR = '0x18',
    MII_PHYADDR = '0x19',
    MII_RESV2 = '0x1a',
    MII_TPISTATUS = '0x1b',
    MII_NCONFIG = '0x1c',
    BMCR_RESV = '0x003f',
    BMCR_SPEED1000 = '0x0040',
    BMCR_CTST = '0x0080',
    BMCR_FULLDPLX = '0x0100',
    BMCR_ANRESTART = '0x0200',
    BMCR_ISOLATE = '0x0400',
    BMCR_PDOWN = '0x0800',
    BMCR_ANENABLE = '0x1000',
    BMCR_SPEED100 = '0x2000',
    BMCR_LOOPBACK = '0x4000',
    BMCR_RESET = '0x8000',
    BMSR_ERCAP = '0x0001',
    BMSR_JCD = '0x0002',
    BMSR_LSTATUS = '0x0004',
    BMSR_ANEGCAPABLE = '0x0008',
    BMSR_RFAULT = '0x0010',
    BMSR_ANEGCOMPLETE = '0x0020',
    BMSR_RESV = '0x00c0',
    BMSR_ESTATEN = '0x0100',
    BMSR_100HALF2 = '0x0200',
    BMSR_100FULL2 = '0x0400',
    BMSR_10HALF = '0x0800',
    BMSR_10FULL = '0x1000',
    BMSR_100HALF = '0x2000',
    BMSR_100FULL = '0x4000',
    BMSR_100BASE4 = '0x8000',
    ADVERTISE_SLCT = '0x001f',
    ADVERTISE_CSMA = '0x0001',
    ADVERTISE_10HALF = '0x0020',
    ADVERTISE_1000XFULL = '0x0020',
    ADVERTISE_10FULL = '0x0040',
    ADVERTISE_1000XHALF = '0x0040',
    ADVERTISE_100HALF = '0x0080',
    ADVERTISE_1000XPAUSE = '0x0080',
    ADVERTISE_100FULL = '0x0100',
    ADVERTISE_1000XPSE_ASYM = '0x0100',
    ADVERTISE_100BASE4 = '0x0200',
    ADVERTISE_PAUSE_CAP = '0x0400',
    ADVERTISE_PAUSE_ASYM = '0x0800',
    ADVERTISE_RESV = '0x1000',
    ADVERTISE_RFAULT = '0x2000',
    ADVERTISE_LPACK = '0x4000',
    ADVERTISE_NPAGE = '0x8000',
    LPA_SLCT = '0x001f',
    LPA_10HALF = '0x0020',
    LPA_1000XFULL = '0x0020',
    LPA_10FULL = '0x0040',
    LPA_1000XHALF = '0x0040',
    LPA_100HALF = '0x0080',
    LPA_1000XPAUSE = '0x0080',
    LPA_100FULL = '0x0100',
    LPA_1000XPAUSE_ASYM = '0x0100',
    LPA_100BASE4 = '0x0200',
    LPA_PAUSE_CAP = '0x0400',
    LPA_PAUSE_ASYM = '0x0800',
    LPA_RESV = '0x1000',
    LPA_RFAULT = '0x2000',
    LPA_LPACK = '0x4000',
    LPA_NPAGE = '0x8000',
    EXPANSION_NWAY = '0x0001',
    EXPANSION_LCWP = '0x0002',
    EXPANSION_ENABLENPAGE = '0x0004',
    EXPANSION_NPCAPABLE = '0x0008',
    EXPANSION_MFAULTS = '0x0010',
    EXPANSION_RESV = '0xffe0',
    ESTATUS_1000_TFULL = '0x2000',
    ESTATUS_1000_THALF = '0x1000',
    NWAYTEST_RESV1 = '0x00ff',
    NWAYTEST_LOOPBACK = '0x0100',
    NWAYTEST_RESV2 = '0xfe00',
    ADVERTISE_1000FULL = '0x0200',
    ADVERTISE_1000HALF = '0x0100',
    CTL1000_AS_MASTER = '0x0800',
    CTL1000_ENABLE_MASTER = '0x1000',
    LPA_1000LOCALRXOK = '0x2000',
    LPA_1000REMRXOK = '0x1000',
    LPA_1000FULL = '0x0800',
    LPA_1000HALF = '0x0400',
    FLOW_CTRL_TX = '0x01',
    FLOW_CTRL_RX = '0x02',
    MII_MMD_CTRL_DEVAD_MASK = '0x1f',
    MII_MMD_CTRL_ADDR = '0x0000',
    MII_MMD_CTRL_NOINCR = '0x4000',
    MII_MMD_CTRL_INCR_RDWT = '0x8000',
    MII_MMD_CTRL_INCR_ON_WT = '0xC000',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

