require 'strict'

-- gate.lua - Implements a "gate" trigger for use by a data mode.

local trig_common = require 'test_engine/modules/trigger/trig_common'
local mt = {}

-- Determines whether the given value has caused the trigger
-- to be on or off.
-- Parameter:
--     'trig_val'  is a sample value from a logical trigger channel
-- Returns:  'true' if trigger is on, false otherwise
function mt:triggered(trig_val)
    if trig_val ~= 0 then
        return true
    else
        return false
    end
end


local m = {}

-- Creates a new trigger object.
-- Parameters:
--   'in_chans'  input channel names
--   'trig_chan' trigger channel name
--   'node_list' list of previously instantiated channel nodes
--   'stanza_list'  list of node metadata stanzas
-- Returns:
--   the new instance object
function m:new(in_chans, trig_chan, node_list, stanza_list)
    assert(in_chans, "missing first parameter")
    assert(trig_chan, "missing second parameter")
    assert(node_list, "missing third parameter")

    local trig_core = setmetatable({}, {__index = mt})

    local trig_obj =
        trig_common:new(in_chans, trig_chan, node_list, trig_core, stanza_list)

    return trig_obj
end

return m
